import os

import flanautils

os.environ |= flanautils.find_environment_variables('../.env')

import asyncio

from flanabot.bots.flana_tele_bot import FlanaTeleBot


async def main():
    flana_tele_bot = FlanaTeleBot()

    await asyncio.gather(
        flana_tele_bot.start()
    )


if __name__ == '__main__':
    asyncio.run(main())



import datetime
from dataclasses import dataclass

from flanautils import DCMongoBase, FlanaBase
from multibot.models.database import db


@dataclass(eq=False)
class PunishmentBase(DCMongoBase, FlanaBase):
    user_id: int = None
    group_id: int = None
    until: datetime.datetime = None
    is_active: bool = True


@dataclass(eq=False)
class Punishment(PunishmentBase):
    collection = db.punishment


@dataclass(eq=False)
class Mute(PunishmentBase):
    collection = db.mute



from dataclasses import dataclass, field

from multibot import Chat as MultiBotChat

from flanabot.models.user import User


@dataclass(eq=False)
class Chat(MultiBotChat):
    DEFAULT_CONFIG = {'auto_clear': False,
                      'auto_covid_chart': True,
                      'auto_currency_chart': True,
                      'auto_delete_original': True,
                      'auto_scraping': True,
                      'auto_weather_chart': True}
    users: list[User] = field(default_factory=list)

    def __post_init__(self):
        super().__post_init__()
        if not self.config:
            self.config = self.DEFAULT_CONFIG



from __future__ import annotations  # todo0 remove in 3.11

import functools
import os
from typing import Callable

import telethon.tl.functions
from flanaapis.weather.constants import WeatherEmoji
from flanautils import Media, MediaType, return_if_first_empty
from multibot import TelegramBot, constants as multibot_constants, find_message, user_client

from flanabot.bots.flana_bot import FlanaBot
from flanabot.models.chat import Chat
from flanabot.models.message import Message
from flanabot.models.user import User


# ---------------------------------------------------------- #
# ----------------------- DECORATORS ----------------------- #
# ---------------------------------------------------------- #


def whitelisted_event(func: Callable) -> Callable:
    @functools.wraps(func)
    @find_message
    async def wrapper(self: FlanaTeleBot, message: Message):
        if message.author.id not in self.whitelist_ids:
            return

        return await func(self, message)

    return wrapper


# ---------------------------------------------------------------------------------------------------- #
# ------------------------------------------ FLANA_TELE_BOT ------------------------------------------ #
# ---------------------------------------------------------------------------------------------------- #
class FlanaTeleBot(TelegramBot, FlanaBot):
    def __init__(self):
        super().__init__(
            api_id=os.environ['TELEGRAM_API_ID'],
            api_hash=os.environ['TELEGRAM_API_HASH'],
            bot_session=os.environ['TELEGRAM_BOT_SESSION'],
            user_session=os.environ['TELEGRAM_USER_SESSION']
        )
        self.whitelist_ids = []

    # ----------------------------------------------------------- #
    # -------------------- PROTECTED METHODS -------------------- #
    # ----------------------------------------------------------- #
    def _add_handlers(self):
        super()._add_handlers()
        self.register_button(self._on_button_press)

    @return_if_first_empty(exclude_self_types='FlanaTeleBot', globals_=globals())
    async def _create_chat_from_telegram_chat(self, telegram_chat: multibot_constants.TELEGRAM_CHAT) -> Chat | None:
        chat = await super()._create_chat_from_telegram_chat(telegram_chat)
        chat.config = Chat.DEFAULT_CONFIG
        return Chat.from_dict(chat.to_dict())

    @return_if_first_empty(exclude_self_types='FlanaTeleBot', globals_=globals())
    async def _create_user_from_telegram_user(self, original_user: multibot_constants.TELEGRAM_USER, group_id: int = None) -> User | None:
        return User.from_dict((await super()._create_user_from_telegram_user(original_user, group_id)).to_dict())

    @user_client
    async def _get_contacts_ids(self) -> list[int]:
        async with self.user_client:
            contacts_data = await self.user_client(telethon.tl.functions.contacts.GetContactsRequest(hash=0))

        return [contact.user_id for contact in contacts_data.contacts]

    @user_client
    async def _update_whitelist(self):
        self.whitelist_ids = [self.owner_id, self.bot_id] + await self._get_contacts_ids()

    # ---------------------------------------------- #
    #                    HANDLERS                    #
    # ---------------------------------------------- #
    @whitelisted_event
    async def _on_button_press(self, message: Message):
        await message.original_event.answer()

        match message.button_text:
            case WeatherEmoji.ZOOM_IN.value:
                message.weather_chart.zoom_in()
            case WeatherEmoji.ZOOM_OUT.value:
                message.weather_chart.zoom_out()
            case WeatherEmoji.LEFT.value:
                message.weather_chart.move_left()
            case WeatherEmoji.RIGHT.value:
                message.weather_chart.move_right()
            case WeatherEmoji.PRECIPITATION_VOLUME.value:
                message.weather_chart.trace_metadatas['rain_volume'].show = not message.weather_chart.trace_metadatas['rain_volume'].show
                message.weather_chart.trace_metadatas['snow_volume'].show = not message.weather_chart.trace_metadatas['snow_volume'].show
            case emoji if emoji in WeatherEmoji.values:
                trace_metadata_name = WeatherEmoji(emoji).name.lower()
                message.weather_chart.trace_metadatas[trace_metadata_name].show = not message.weather_chart.trace_metadatas[trace_metadata_name].show

        message.weather_chart.apply_zoom()
        message.weather_chart.draw()
        message.save()

        image_bytes = message.weather_chart.to_image()
        file = await self._prepare_media_to_send(Media(image_bytes, MediaType.IMAGE))

        try:
            await message.original_object.edit(file=file)
        except telethon.errors.rpcerrorlist.MessageNotModifiedError:
            pass

    @whitelisted_event
    async def _on_inline_query_raw(self, message: Message):
        await super()._on_new_message_raw(message)

    @whitelisted_event
    async def _on_new_message_raw(self, message: Message):
        await super()._on_new_message_raw(message)

    async def _on_ready(self):
        await super()._on_ready()
        await self._update_whitelist()

    # -------------------------------------------------------- #
    # -------------------- PUBLIC METHODS -------------------- #
    # -------------------------------------------------------- #



from __future__ import annotations  # todo0 remove in 3.11

from dataclasses import dataclass, field
from typing import Iterable

from flanautils import Media, OrderedSet
from multibot import Message as MultiBotMessage

from flanabot.models.chat import Chat
from flanabot.models.user import User
from flanabot.models.weather_chart import WeatherChart


@dataclass(eq=False)
class Message(MultiBotMessage):
    author: User = None
    mentions: Iterable[User] = field(default_factory=list)
    chat: Chat = None
    replied_message: Message = None
    weather_chart: WeatherChart = None
    song_infos: OrderedSet[Media] = field(default_factory=OrderedSet)



import asyncio
import os

import discord
from multibot import DiscordBot

from flanabot import constants
from flanabot.bots.flana_bot import FlanaBot
from flanabot.exceptions import BadRoleError, UserDisconnectedError
from flanabot.models import User

HEAT_NAMES = [
    'Canal Congelado',
    'Canal Fresquito',
    'Canal Templaillo',
    'Canal Calentito',
    'Canal Caloret',
    'Canal Caliente',
    'Canal Olor a Vasco',
    'Verano Cordobés al Sol',
    'Canal Ardiendo',
    'abrid las putas ventanas y traed el extintor',
    'Canal INFIERNO',
    'La Palma 🌋'
]
HOT_CHANNEL_ID = 493530483045564417
ROLES = {
    'Administrador': 387344390030360587,
    'Carroñero': 493523298429435905,
    'al lol': 881238165476741161,
    'Persona': 866046517998387220,
    'Castigado': 877662459568209921,
    'Bot': 493784221085597706
}


# ---------------------------------------------------------------------------------------------------- #
# ------------------------------------------ FLANA_DISC_BOT ------------------------------------------ #
# ---------------------------------------------------------------------------------------------------- #
class FlanaDiscBot(DiscordBot, FlanaBot):
    def __init__(self):
        super().__init__(os.environ['DISCORD_BOT_TOKEN'])
        self.heating = False
        self.heat_level = 0

    # ----------------------------------------------------------- #
    # -------------------- PROTECTED METHODS -------------------- #
    # ----------------------------------------------------------- #
    def _add_handlers(self):
        super()._add_handlers()
        self.bot_client.add_listener(self._on_voice_state_update, 'on_voice_state_update')

    async def _heat_channel(self, channel: discord.VoiceChannel):
        while True:
            await asyncio.sleep(constants.HEAT_PERIOD_SECONDS)

            if channel.members:
                if self.heat_level == len(HEAT_NAMES) - 1:
                    return
                self.heat_level += 0.5
            elif not channel.members:
                if not self.heat_level:
                    return
                self.heat_level -= 0.5
            else:
                continue

            await channel.edit(name=HEAT_NAMES[int(self.heat_level)])

    async def _mute(self, user_id: int, group_id: int):
        user = await self.get_user(user_id, group_id)
        try:
            await user.original_object.edit(mute=True)
        except discord.errors.HTTPException:
            raise UserDisconnectedError

    async def _punish(self, user_id: int, group_id: int):
        user = await self.get_user(user_id, group_id)
        try:
            await user.original_object.remove_roles(self._find_role_by_id(ROLES['Persona'], user.original_object.guild.roles))
            await user.original_object.add_roles(self._find_role_by_id(ROLES['Castigado'], user.original_object.guild.roles))
        except AttributeError:
            raise BadRoleError(str(self._punish))

    async def _unmute(self, user_id: int, group_id: int):
        user = await self.get_user(user_id, group_id)
        try:
            await user.original_object.edit(mute=False)
        except discord.errors.HTTPException:
            raise UserDisconnectedError

    async def _unpunish(self, user_id: int, group_id: int):
        user = await self.get_user(user_id, group_id)
        try:
            await user.original_object.remove_roles(self._find_role_by_id(ROLES['Castigado'], user.original_object.guild.roles))
            await user.original_object.add_roles(self._find_role_by_id(ROLES['Persona'], user.original_object.guild.roles))
        except AttributeError:
            raise BadRoleError(str(self._unpunish))

    # ---------------------------------------------- #
    #                    HANDLERS                    #
    # ---------------------------------------------- #
    async def _on_voice_state_update(self, _: discord.Member, before: discord.VoiceState, after: discord.VoiceState):
        if getattr(before.channel, 'id', None) == HOT_CHANNEL_ID:
            channel = before.channel
        elif getattr(after.channel, 'id', None) == HOT_CHANNEL_ID:
            channel = after.channel
        else:
            return

        if not self.heating:
            self.heating = True
            await self._heat_channel(channel)
            self.heating = False

    # -------------------------------------------------------- #
    # -------------------- PUBLIC METHODS -------------------- #
    # -------------------------------------------------------- #
    async def is_deaf(self, user_id: int, group_id: int) -> bool:
        user = await self.get_user(user_id, group_id)
        return user.original_object.voice.deaf

    async def is_muted(self, user_id: int, group_id: int) -> bool:
        user = await self.get_user(user_id, group_id)
        return user.original_object.voice.mute

    @staticmethod
    def is_self_deaf(user: User) -> bool:
        return user.original_object.voice.self_deaf

    @staticmethod
    def is_self_muted(user: User) -> bool:
        return user.original_object.voice.self_mute



import datetime

import flanautils

AUTO_WEATHER_EVERY = datetime.timedelta(hours=6)
CHECK_MESSAGE_EVERY_SECONDS = datetime.timedelta(days=1).total_seconds()
CHECK_MUTES_EVERY_SECONDS = datetime.timedelta(hours=1).total_seconds()
CHECK_PUNISHMENTS_EVERY_SECONDS = datetime.timedelta(hours=1).total_seconds()
HEAT_PERIOD_SECONDS = datetime.timedelta(minutes=15).total_seconds()
MAX_PLACE_QUERY_LENGTH = 50
PUNISHMENT_INCREMENT_EXPONENT = 6
PUNISHMENTS_RESET = datetime.timedelta(weeks=6 * flanautils.WEEKS_IN_A_MONTH)
RECOVERY_DELETED_MESSAGE_BEFORE = datetime.timedelta(hours=1)
TIME_THRESHOLD_TO_MANUAL_UNMUTE = datetime.timedelta(days=3)
TIME_THRESHOLD_TO_MANUAL_UNPUNISH = datetime.timedelta(days=3)
SCRAPING_MESSAGE_WAITING_TIME = 0.1

BAD_PHRASES = (
    'Cállate ya anda.',
    '¿Quién te ha preguntado?',
    '¿Tú eres así o te dan apagones cerebrales?',
    'Ante la duda mi dedo corazón te saluda.',
    'Enjoy cancer brain.',
    'Calla noob.',
    'Hablas tanta mierda que tu culo tiene envidia de tu boca.',
    'jAJjajAJjajAJjajAJajJAJajJA',
    'enjoy xd',
    'Reported.',
    'Baneito pa ti en breve.',
    'Despídete de tu cuenta.',
    'Flanagan es más guapo que tú.',
    'jajaj',
    'xd',
    'Hay un concurso de hostias y tienes todas las papeletas.',
    '¿Por qué no te callas?',
    'Das penilla.',
    'Deberían hacerte la táctica del C4.',
    'Te voy romper las pelotas.',
    'Más tonto y no naces.',
    'Eres más tonto que peinar bombillas.',
    'Eres más tonto que pellizcar cristales.',
    'Eres más malo que pegarle a un padre.'
)

BYE_PHRASES = ('Adiós.', 'adieu', 'adio', 'adioh', 'adios', 'adió', 'adiós', 'agur', 'bye', 'byyeeee', 'chao',
               'hasta la vista', 'hasta luego', 'hasta nunca', ' hasta pronto', 'hasta la próxima',
               'nos vemos', 'taluego')

HELLO_PHRASES = ('alo', 'aloh', 'buenas', 'Hola.', 'hello', 'hey', 'hi', 'hola', 'holaaaa', 'holaaaaaaa', 'ola',
                 'ola k ase', 'pa ti mi cola', 'saludos')

KEYWORDS = {
    'activate': ('activa', 'activar', 'activate', 'deja', 'dejale', 'devuelve', 'devuelvele', 'enable', 'encender',
                 'enciende', 'habilita', 'habilitar'),
    'bye': ('adieu', 'adio', 'adiooooo', 'adios', 'agur', 'buenas', 'bye', 'cama', 'chao', 'farewell', 'goodbye',
            'hasta', 'luego', 'noches', 'pronto', 'taluego', 'taluegorl', 'tenga', 'vemos', 'vista', 'voy'),
    'change': ('alter', 'alternar', 'alternate', 'cambiar', 'change', 'default', 'defecto', 'edit', 'editar',
               'exchange', 'modificar', 'modify', 'permutar', 'predeterminado', 'shift', 'swap', 'switch', 'turn',
               'vary'),
    'config': ('ajustar', 'ajuste', 'ajustes', 'automatico', 'automatic', 'config', 'configs', 'configuracion',
               'configuration', 'default', 'defecto', 'setting', 'settings'),
    'covid_chart': ('case', 'caso', 'contagiado', 'contagio', 'corona', 'coronavirus', 'covid', 'covid19', 'death',
                    'disease', 'enfermedad', 'enfermos', 'fallecido', 'incidencia', 'jacovid', 'mascarilla', 'muerte',
                    'muerto', 'pandemia', 'sick', 'virus'),
    'currency_chart': ('argentina', 'bitcoin', 'cardano', 'cripto', 'crypto', 'criptodivisa', 'cryptodivisa',
                       'cryptomoneda', 'cryptocurrency', 'currency', 'dinero', 'divisa', 'ethereum', 'inversion',
                       'moneda', 'pasta'),
    'date': ('ayer', 'de', 'domingo', 'fin', 'finde', 'friday', 'hoy', 'jueves', 'lunes', 'martes', 'mañana',
             'miercoles', 'monday', 'pasado', 'sabado', 'saturday', 'semana', 'sunday', 'thursday', 'today', 'tomorrow',
             'tuesday', 'viernes', 'wednesday', 'week', 'weekend', 'yesterday'),
    'deactivate': ('apaga', 'apagar', 'deactivate', 'deactivate', 'desactivar', 'deshabilita', 'deshabilitar',
                   'disable', 'forbids', 'prohibe', 'quita', 'remove', 'return'),
    'hello': ('alo', 'aloh', 'buenas', 'dias', 'hello', 'hey', 'hi', 'hola', 'holaaaaaa', 'ola', 'saludos', 'tardes'),
    'help': ('ayuda', 'help'),
    'mute': ('calla', 'calle', 'cierra', 'close', 'mute', 'mutea', 'mutealo', 'noise', 'ruido', 'shut', 'silence',
             'silencia'),
    'negate': ('no', 'ocurra', 'ocurre'),
    'permission': ('permiso', 'permission'),
    'punish': ('acaba', 'aprende', 'ataca', 'atalo', 'azota', 'boss', 'castiga', 'castigo', 'condena', 'controla',
               'destroy', 'destroza', 'duro', 'ejecuta', 'enseña', 'escarmiento', 'execute', 'finish', 'fuck', 'fusila',
               'hell', 'humos', 'infierno', 'jefe', 'jode', 'learn', 'leccion', 'lesson', 'manda', 'purgatorio',
               'sancion', 'shoot', 'teach', 'termina', 'whip'),
    'reset': ('recover', 'recovery', 'recupera', 'reinicia', 'reset', 'resetea', 'restart'),
    'scraping': ('api', 'aqui', 'busca', 'contenido', 'content', 'descarga', 'descargar', 'download', 'envia', 'habia',
                 'media', 'redes', 'scrap', 'scraping', 'search', 'send', 'social', 'sociales', 'tenia', 'video',
                 'videos'),
    'show': ('actual', 'enseña', 'estado', 'how', 'muestra', 'show', 'como'),
    'song_info': ('aqui', 'cancion', 'data', 'datos', 'info', 'informacion', 'information', 'llama', 'media', 'name',
                  'nombre', 'sonaba', 'sonando', 'song', 'sono', 'sound', 'suena', 'title', 'titulo',
                  'video'),
    'sound': ('hablar', 'hable', 'micro', 'microfono', 'microphone', 'sonido', 'sound', 'talk', 'volumen'),
    'thanks': ('gracia', 'gracias', 'grasia', 'grasias', 'grax', 'thank', 'thanks', 'ty'),
    'unmute': ('desilencia', 'desmutea', 'desmutealo', 'unmute'),
    'unpunish': ('absolve', 'forgive', 'innocent', 'inocente', 'perdona', 'spare'),
    'weather_chart': ('atmosfera', 'atmosferico', 'calle', 'calor', 'caloret', 'clima', 'climatologia', 'cloud',
                      'cloudless', 'cloudy', 'cold', 'congelar', 'congelado', 'denbora', 'despejado', 'diluvio', 'frio',
                      'frost', 'hielo', 'humedad', 'llover', 'llueva', 'llueve', 'lluvia', 'nevada', 'nieva', 'nieve',
                      'nube', 'nubes', 'nublado', 'meteorologia', 'rain', 'snow', 'snowfall', 'snowstorm', 'sol',
                      'solano', 'storm', 'sun', 'temperatura', 'tempo', 'tiempo', 'tormenta', 'ventisca', 'weather',
                      'wetter')
}

RECOVER_PHRASES = (
    'No hay nada que recuperar.',
    'Ya lo he recuperado y enviado, así que callate ya.',
    'Ya lo he recuperado y enviado, así que mejor estás antento antes de dar por culo.',
    'Ya lo he recuperado y enviado, no lo voy a hacer dos veces.',
    'Ya lo he recuperado y enviado. A ver si leemos más y jodemos menos.',
    'Ya lo he reenviado.'
)

SCRAPING_PHRASES = ('Analizando...', 'Buscando...', 'Hackeando internet... 👀', 'Rebuscando en la web...',
                    'Robando cosas...', 'Scrapeando...', 'Scraping...')
